/**
 * Module dependencies.
 */

var nodes = require('jade').nodes
  , Node = nodes.Node;

/**
 * Serialize function.
 *
 */

function serialize(node){
  var ret = {};
  
  if (node instanceof nodes.Block){
    ret.__type = 'block';
  } else if (node instanceof nodes.Code){
    ret.__type = 'code';
  } else if (node instanceof nodes.Doctype){
    ret.__type = 'doctype';
  } else if (node instanceof nodes.Filter){
    ret.__type = 'filter';
  } else if (node instanceof nodes.Tag){
    ret.__type = 'tag';
  } else if (node instanceof nodes.Text){
    ret.__type = 'text';
  }
  
  node.forEach(function(item){
    if (!('__items' in ret))
      ret.__items = [];
    ret.__items.push(item instanceof Node ? serialize(item) : item);
  });
  
  for (var i in node){
    if (node.hasOwnProperty(i) && Number(i) != i && i !== 'length' && i !== 'line'){
      if (node[i] instanceof Node){
        ret[i] = serialize(node[i]);
      } else {
        ret[i] = node[i];
      }
    }
  }
  
  return ret;
};

/**
 * Deserialize function
 *
 */

function deserialize(node){
  var type = node.__type
    , obj;

  type = type[0].toUpperCase() + type.substr(1);
  obj = new nodes[type];

  if ('__items' in node){
    node.__items.forEach(function(item, i){
      if (typeof item == 'object' && '__type' in item)
        node.__items[i] = deserialize(item);
    });
    obj.push.apply(obj, node.__items);
  }
    
  for (var i in node){
    if (node.hasOwnProperty(i)){
      if (i !== '__type'){
        if (typeof node[i] == 'object' && '__type' in node[i]){
          obj[i] = deserialize(node[i]);
        } else {
          obj[i] = node[i];
        }
      }
    }
  }
  
  return obj;
};

module.exports = {
  
  /**
   * Serializes the node into a simple object construct
   *
   * @api public
   */
   
  serialize: function(node){
    return serialize(node);
  },
  
  /**
   * Deserializes an object into a valid compiled Jade node
   *
   * @api public
   */
  
  deserialize: function(node){
    return deserialize(node);
  }
  
};