var jade = require('jade')
  , nodes = jade.nodes
  , fs = require('fs')
  , serialize = require('jade-serial').serialize
  , deserialize = require('jade-serial').deserialize
  , Parser = require('jade');

function Compiler(node){
  this.node = node;
};

function parse(fixture, fn){
  fs.readFile(__dirname + '/fixtures/' + fixture, 'utf8', function(err, str){
    if (err) return fn(err);
    var _compiler
      , compiler = function(node){
          _compiler = node;
        };
    compiler.prototype.compile = function(){};
    jade.render(str, {compiler: compiler});
    fn(_compiler[0]);
  });
};

module.exports = {
  
  'test serializing a text node': function(assert){
    parse('text.jade', function(node){
      var s = serialize(node);
      assert.ok(s.__type == 'text');
      assert.ok(s.__items[0] == 'Text node')
    })
  },
  
  'test serializing a block node': function(assert){
    parse('block.jade', function(node){
      var s = serialize(node);
      assert.ok(s.__type == 'code');
      assert.ok(s.val == ' if (1==1)');
      assert.ok(s.block.__type == 'block');
    });
  },
  
  'test serializing a doctype': function(assert){
    parse('doctype.jade', function(node){
      assert.ok(serialize(node).__type == 'doctype');
    });
  },
  
  'test serializing a filter': function(assert){
    parse('filter.jade', function(node){
      assert.ok(serialize(node).__type == 'filter');
    });
  },
  
  'test serializing a tag': function(assert){
    parse('tag.jade', function(node){
      var s = serialize(node);
      assert.ok(s.__type == 'tag');
      assert.ok(s.attrs[0]['name'] == 'href');
      assert.ok(s.attrs[0]['val'] == " '#'");
    });
  },
  
  'test serializing a tag with text': function(assert){
    parse('tag-text.jade', function(node){
      var s = serialize(node);
      assert.ok(s.__type == 'tag');
      assert.ok(s.text.__type == 'text');
      assert.ok(s.text.__items[0] == ' Hello');
    });
  },
  
  'test deserializing a text node': function(assert){
    var obj = deserialize({
      __type: 'text',
      __items: ['Line 1', 'Line 2']
    });
    
    assert.ok(obj instanceof nodes.Text);
    assert.ok(obj.length == 2);
    assert.ok(obj[0] == 'Line 1');
    assert.ok(obj[1] == 'Line 2');
  },
  
  'test deserializing a tag': function(assert){
    var obj = deserialize({
      __type: 'tag',
      text: { __type: 'text' }
    });
    assert.ok(obj instanceof nodes.Tag);
    assert.ok(obj.text instanceof nodes.Text);
  },
  
  'test deserializing nested nodes in items': function(assert){
    var obj = deserialize({
      __type: 'block',
      __items: [ { __type: 'tag', name: 'a'} ]
    });
    assert.ok(obj instanceof nodes.Block);
  }
  
};